      FUNCTION AQUAD(RULE,N,F,A,B,TOL,MAXLEV,SING,ERREST)
C
C
C   THIS   FUNCTION  SUBPROGRAM  APPROXIMATES  THE  VALUE  OF  A
C   DEFINITE INTEGRAL I(F;A,B)  OF  A  FUNCTION  F(X)  OVER  THE
C   INTERVAL  (A,B) USING AN ADAPTIVE QUADRATURE PROCEDURE.  THE
C   UNDERLYING QUADRATURE RULE TO BE USED IS  SPECIFIED  BY  THE
C   PARAMETERS RULE AND N.
C
C   THE ROUTINE SUCCESSIVELY BISECTS THE INTERVAL OF INTEGRATION
C   AND   APPLIES   THE  SPECIFIED  QUADRATURE  RULE  OVER  EACH
C   SUBINTERVAL.  A SUBINTERVAL (XL,XR) IS "ACCEPTED" IF
C
C       |Q(F;XL,XR) - ( Q(F;XL,XM) + Q(F;XM,XL) )|
C
C                               <= TOL*FAC*|XR - XL|/|B - A|,
C
C   WHERE
C     Q(F;XL,XR)  DENOTES THE APPROXIMATE VALUE OF THE  INTEGRAL
C       I(F;XL,XR), OF  F(X)  OVER (XR,XL);
C     XM  IS THE MIDPOINT OF (XL,XR), AND
C     FAC  IS A SCALING FACTOR, DEPENDENT ON N AND THE  TYPE  OF
C       RULE.  IN PARTICULAR,
C         FAC =  2**(N+2)-1     FOR NEWTON-COTES RULES,
C             =  2**(2*N)-1     FOR GAUSS-LEGENDRE RULES.
C       THE  CORRECT  VALUE  OF FAC IS PROVIDED AUTOMATICALLY BY
C       THE SUBROUTINE SPECIFIED BY THE VALUE OF RULE.
C
C   IF THIS  TEST  FAILS,  THEN  (XL,XR)  IS  BISECTED  AND  THE
C   PROCEDURE  IS REPEATED ON THE LEFT HALF.  WHEN A SUBINTERVAL
C   IS ACCEPTED, THE ROUTINE GOES TO THE NEXT SUBINTERVAL TO THE
C   RIGHT,AND SO ON.
C
C   THE MAXIMUM LEVEL OF BISECTION CAN BE CONTROLLED THROUGH THE
C   PARAMETER  MAXLEV.   IF  THE  MAXIMUM LEVEL OF BISECTION HAS
C   BEEN REACHED FOR A SUBINTERVAL (XL,XR)  BUT  THE  ACCEPTANCE
C   TEST  STILL FAILS THEN THE MIDPOINT XM IS RECORDED IN "SING"
C   AND THE ROUTINE WILL CONTINUE THE INTEGRATION WITH THE  NEXT
C   SUBINTERVAL  TO  THE  RIGHT.  THE VALUE OF THE INTEGRAL OVER
C   THE "FAILED" SUBINTERVAL IS TAKEN TO BE Q(F;XL,XR).
C
C
C   CALLING SEQUENCE:  APPROX = AQUAD(RULE,N,F,A,B,TOL,MAXLEV,
C                                      SING,ERREST)
C
C   PARAMETERS:
C
C   ON ENTRY:
C
C        RULE   - SUBROUTINE NAME
C                  A  ROUTINE,  RULE(N,XL,XR,P,W), FOR SUPPLYING
C                  THE QUADRATURE POINTS XI AND THE  WEIGHTS  WI
C                  RELATIVE   TO   THE   INTERVAL   (XL,XR)   OF
C                  INTEGRATION.   THE   FOLLOWING   VALUES  WILL
C                  ACCESS  THE  RULES THAT HAVE BEEN IMPLEMENTED
C                  IN THE PACKAGE:
C                    RULE = NCQ     - NEWTON-COTES RULES;
C                         = GLEGQ   - GAUSS-LEGENDRE RULES.
C                  NOTE:  SINCE  GAUSS-LAGUERRE  RULES  ARE  NOT
C                  SUITABLE FOR ADAPTIVE QUADRATURE, THE ROUTINE
C                  GLAGQ SHOULD NOT BE USED WITH AQUAD.
C                  A USER-SUPPLIED SUBROUTINE IMPLEMENTATION  OF
C                  A  QUADRATURE  RULE CAN ALSO BE USED.  SUCH A
C                  ROUTINE MUST, OF COURSE, CONFORM TO THE ABOVE
C                  CALLING SEQUENCE.
C
C        N      - INTEGER
C                  THE  NUMBER  OF  QUADRATURE POINTS TO BE USED
C                  IN EACH  SUBINTERVAL  (XL,XR).  LIMITS  ON  N
C                  WHEN RULE =
C                     NCQ   :  1 <= N <= 21;
C                     GLEGQ :  2 <= N <= 20.
C
C        F      - REAL
C                  A USER-SUPPLIED  FUNCTION  SUBPROGRAM,  F(X),
C                  FOR  EVALUATING  THE INTEGRAND AT ANY POINT X
C                  IN (A,B).
C
C        A,B    - REAL
C                  THE LEFT AND RIGHT ENDPOINTS OF THE  INTERVAL
C                  OF INTEGRATION.
C
C        TOL    - REAL
C                  USED  IN  THE  CRITERION  FOR   ACCEPTING   A
C                  SUBINTERVAL.
C
C        MAXLEV - INTEGER
C                  THE MAXIMUM LEVEL OF BISECTION ALLOWED.   THE
C                  SMALLEST  POSSIBLE  LENGTH  OF  A SUBINTERVAL
C                  WILL  BE  (B - A)/2**MAXLEV,  WHERE
C                  0 <= MAXLEV <= 30.
C
C   ON RETURN:
C
C        AQUAD  - REAL
C                  THE APPROXIMATE VALUE OF THE INTEGRAL.
C
C        TOL    -  THE FRACTION OF THE  INTERVAL OF  INTEGRATION
C                  (A,B) ON  WHICH  THE  SUBINTERVALS  WERE  NOT
C                  ACCEPTED.  IF TOL  >  0, THERE HAVE BEEN SOME
C                  DIFFICULTIES.  THE USER SHOULD CHECK WHETHER
C                  - THE INTEGRAND HAS A  SINGULARITY
C                    (AT X = SING), OR ELSE
C                  - THE INPUT VALUES OF TOL AND/OR MAXLEV  WERE
C                    TOO  SMALL  FOR  ACCEPTANCE TO OCCUR BEFORE
C                    THE MAXIMUM NUMBER OF BISECTIONS WAS DONE.
C
C        MAXLEV -  THE NUMBER  OF  SUBINTERVALS INTO  WHICH  THE
C                  INTERVAL OF INTEGRATION (A,B) WAS DIVIDED.
C
C        SING   - REAL
C                  WHEN TOL > 0, SING WILL BE  THE  MIDPOINT  OF
C                  THE  SUB-INTERVAL  THAT  IS CLOSEST TO  B AND
C                  FOR WHICH  THE  ACCEPTANCE  CRITERION  FAILED
C                  AFTER MAXLEV LEVELS OF BISECTION WERE DONE.
C
C        ERREST - REAL
C                  AN ESTIMATE OF THE  ERROR  IN  THE  VALUE  OF
C                  AQUAD AS AN APPROXIMATION FOR I(F;A,B).
C
C
C   SAMPLE CALLING PROGRAM:
C
C  C  *************************
C  C  *
C  C  *  THIS  PROGRAM  APPROXIMATES THE VALUE OF I(F;A,B) USING
C  C  *  ADAPTIVE QUADRATURE BASED ON A 6-POINT GAUSS-LEGENDRE
C  C  *  RULE.
C  C  *
C  C  ************************
C  C
C        REAL A,B,TOL,APPROX,SING,ERREST
C        EXTERNAL F,GLEGQ
C        READ (5,*) A,B
C        TOL = 1.E-5
C        MAXLEV = 30
C        APPROX = AQUAD(GLEGQ,6,F,A,B,TOL,MAXLEV,SING,ERREST)
C        IF (TOL .NE. 0.E0) WRITE (6,*) 'ROUTINE FAILED. TOL = ',TOL,
C       +                               '  SING = ',SING
C        WRITE (6,*) 'APPROXIMATE VALUE OF INTEGRAL = ',APPROX,
C       +            ' ERROR ESTIMATE = ',ERREST,
C       +            ' # SUBINTERVALS USED = ',MAXLEV
C        STOP
C        END
C
C        FUNCTION F(X)
C        F = ... EXPRESSION TO EVALUATE THE INTEGRAND
C        RETURN
C        END
C
C
      REAL QRIGHT(30),XSAVE(30),F,A,B,TOL,SING,ERREST,FAC,SUM,CORR,
     *     FLAG,XL,XR,TOLPU,QPREV,XM,STEP,QLEFT,QNOW,QDIFF,TEMP
      INTEGER LSAVE(30)
      EXTERNAL RULE,F
C
C      ***************
C      *
C      * STRAIGHT INTEGRATION IF MAXLEV = 0.
C      *
C      ***************
C
         IF (MAXLEV .NE. 0) GO TO 4
              AQUAD = QUAD(RULE,N,F,A,B)
      RETURN
C
C      ***************
C      *
C      * OBTAIN  THE  APPROPRIATE SCALING FACTOR  FAC  (FOR THE
C      * CONVERGENCE CRITERION AND THE ERROR ESTIMATE) FROM THE
C      * SUBROUTINE DEFINED BY RULE.
C      *
C      ***************
C
   4     NF = -N
         CALL RULE(NF,A,B,XSAVE,QRIGHT)
         FAC = XSAVE(1)
C
C      ***************
C      *
C      * INITIALIZATION AND CALCULATIONS FOR THE ZERO-TH LEVEL.
C      *
C      ***************
C
   8     SUM = 0.0E0
         CORR = 0.0E0
         ERREST = 0.0E0
         FLAG = 0.0E0
         SING = 0.0E0
         ICOUNT = 0
         XL = A
         XR = B
         TOLPU = TOL*FAC/(B - A)
         QPREV = QUAD(RULE,N,F,A,B)
C
C      ***************
C      *
C      * SUCCESSIVELY BISECT THE SUBINTERVALS UNTIL EITHER THE MAXIMUM
C      * ALLOWED LEVEL IS REACHED OR THE INTEGRAL ON A SUBINTERVAL
C      * AGREES WITH THE SUM OF THE INTEGRALS ON ITS 2 HALVES.
C      *
C      ***************
C
         LEV = 1
         LEVEL = 1
  10     XM = 0.5E0*(XL + XR)
         STEP = XR - XL
         QLEFT = QUAD(RULE,N,F,XL,XM)
         QRIGHT(LEV) = QUAD(RULE,N,F,XM,XR)
         QNOW = QLEFT + QRIGHT(LEV)
         QDIFF = QNOW - QPREV
C
C      ***************
C      *
C      * TEST FOR INTERVAL CONVERGENCE AND CHECK IF THE MAXIMUM LEVEL
C      * IS REACHED.
C      *
C      ***************
C
         IF (ABS(QDIFF) .LE. TOLPU*STEP) GO TO 30
              IF (LEVEL .GE. MAXLEV) GO TO 20
                   XSAVE(LEV) = XR
                   LSAVE(LEV) = LEVEL
                   XR = XM
                   QPREV = QLEFT
                   LEVEL = LEVEL + 1
                   LEV = LEV + 1
                   GO TO 10
C
C      ***************
C      *
C      * MAXIMUM LEVEL WAS REACHED. ACCUMULATE THE TOTAL LENGTH OF THE
C      * SUBINTERVALS ON WHICH THE CONVERGENCE FAILED, AND RECORD THE
C      * LOCATION OF THE SINGULARITY.
C      *
C      ***************
C
  20          FLAG = FLAG + STEP
              SING = XM
C
C      ***************
C      *
C      * THE INTEGRAL ON THE INTERVAL CONVERGED OR THE MAXIMUM ALLOWED
C      * LEVEL WAS REACHED.  ADD CONTRIBUTION INTO THE RUNNING SUMS.
C      *
C      ***************
C
  30     SUM = SUM + QNOW
         CORR = CORR + QDIFF
         ERREST = ERREST + ABS(QDIFF)
         ICOUNT = ICOUNT + 1
         IF (LEV .LE. 1) GO TO 40
              LEV = LEV - 1
              XL = XR
              XR = XSAVE(LEV)
              QPREV = QRIGHT(LEV)
              LEVEL = LSAVE(LEV) + 1
              GO TO 10
C
C      ***************
C      *
C      * ADD THE CORRECTION TERM FROM THE FIRST STEP OF RICHARDSON
C      * EXTRAPOLATION.  MAKE SURE ERREST IS NOT LESS THAN ROUNDOFF
C      * LEVEL.
C      *
C      ***************
C
  40     AQUAD = SUM + CORR/FAC
         ERREST = ERREST/FAC
         TOL = FLAG
         MAXLEV = ICOUNT*2
         IF (ERREST .EQ. 0.0E0) RETURN
  50     TEMP = ABS(AQUAD) + ERREST
         IF (TEMP .NE. ABS(AQUAD)) RETURN
         ERREST = 2.0E0*ERREST
         GO TO 50
      END
